==> ruby-borges-0.1.1.tbz: lib/ruby/site_ruby/1.8/borges/Application.rb <==
module Borges

##
# Register a new application that starts with entry_point or at a block

def self.register_application(name, entry_point = nil, &block)
  Application.register(name, entry_point || block)
end

##
#

class Application

  Apps = {}
  
  attr_accessor :entry_point
  attr_reader :sessions

  ##
  # Register an Application

  def self.register(name, entry_point)
    Apps[name] ||= self.new(entry_point)
  end
  
  ##
  # Pass a request off to the appropriate Application

  def self.handle_request(request, response)
    response.application_key = request.application_key
    if app = Apps[request.application_key]
      app.handle_request(request, response)
    else
      response.not_found("No such application: #{request.application_key}",
        request)
    end
  end

  ##
  #

  def initialize(entry_point)
    @sessions = Cache.new
    @entry_point = entry_point
  end
  
  ##
  # Create a new session for this request if the session_key does
  # not exist
  #
  # Otherwise, pass the request to the session and set the response's
  # session_key to the request's session_key

  def handle_request(request, response)
    if request.session_key
      session = @sessions[request.session_key]
      response.session_key = request.session_key
    end
    session ||= new_session(request, response)
    session.handle_request(request, response)
  end
  
  private

  ##
  # Create a new session, and set the response's session_key

  def new_session(request, response)
    session = Session.new(self)
    key = @sessions.store(session)
    response.session_key = key
    session
  end

  def start(&block)
    @entry_point = block
  end
end

end

==> ruby-borges-0.2.0.tbz: lib/ruby/site_ruby/1.6/borges/Application.rb <==
module Borges

##
# Register a new application that starts with entry_point or at a block

def self.register_application(name, entry_point = nil, &block)
  Application.register(name, entry_point || block)
end

##
# Accessing '/' will redirect to application +name+

def self.root_application=(name)
  Application.root_application=(name)
end

##
#

class Application

  Apps = {}
  
  attr_accessor :entry_point
  attr_reader :sessions

  ##
  # Register an Application

  def self.register(name, entry_point)
    Apps[name] ||= self.new(entry_point)
  end
  
  ##
  # Pass a request off to the appropriate Application
  #
  # If the request is for '/' and @@root_application exists, redirect
  # ourselves over there.

  def self.handle_request(request, response)
    response.application_key = request.application_key
    if app = Apps[request.application_key] then
      app.handle_request(request, response)

    elsif request.application_key == "" then
      response.redirect(301, "/#{@@root_application}")

    else
      response.not_found(
        "No such application: #{request.application_key.inspect}\n",
        request)
    end
  end

  ##
  # Set the root application

  def self.root_application=(name)
    @@root_application = name
  end

  ##
  #

  def initialize(entry_point, max_sessions = 10)
    @entry_point = entry_point
    @sessions = LRUCache.new max_sessions
  end
  
  ##
  # Create a new session for this request if the session_key does
  # not exist
  #
  # Otherwise, pass the request to the session and set the response's
  # session_key to the request's session_key

  def handle_request(request, response)
    if request.session_key
      session = @sessions[request.session_key]
      response.session_key = request.session_key
    end

    session ||= new_session(response)
    session.handle_request(request, response)
  end
  
  private

  ##
  # Create a new session, and set the response's session_key

  def new_session(response)
    session = Session.new(self)
    key = @sessions.store(session)
    response.session_key = key
    session
  end

  def start(&block)
    @entry_point = block
  end
end

end
